#ifndef VEHICLE_H
#define VEHICLE_H

#include "BulletDynamics/Vehicle/btRaycastVehicle.h"

#include "LinearMath/btVector3.h"
#include "LinearMath/btMatrix3x3.h"
#include "LinearMath/btTransform.h"
#include "LinearMath/btQuickprof.h"
#include "LinearMath/btAlignedObjectArray.h"
#include "btBulletDynamicsCommon.h"

#include <Aurora/Graphics/models/ModelObj2.h>
#include <Aurora/System/SystemManager.h>

using namespace Aurora::Graphics;
using namespace Aurora::System;


class Vehicle
{
private:
	
	btRigidBody*		mChassisRigidBody;
	btCompoundShape*	mCompoundShape;
	btCollisionShape*	mChassisShape;
	btCollisionShape*	mSimplifiedConvexShape;

	btCollisionShape* m_wheelShape;

	btRaycastVehicle::btVehicleTuning	mVehicleTuning;
	btVehicleRaycaster*					mVehicleRaycaster;
	btRaycastVehicle*					mRaycastVehicle;

	btDynamicsWorld*					mDynamicsWorld;

	// Stores vehicle direction
	btVector3	mForward;

	ModelObj2* _carModel;
	ModelObj2* _wheelModel;

	// variables for motion
	float	mSteeringValue;
	float	mMaxEngineForce;//this should be engine/velocity dependent
	float	mMaxBreakingForce;
	float	mSteeringIncrement;
	float	mSteeringClamp;
	float	mEngineForce;
	float	mBreakingForce;

	// car characteristics
	float	mWheelRadius;
	float	mSuspensionStiffness;
	float	mSuspensionDamping;
	float	mSuspensionCompression;
	float	mWheelFriction;
	float	mRollInfluence;
	float	mWheelWidth;

	float	mSize;

	btRigidBody* CreateRigidBody(float mass, const btTransform& startTransform, btCollisionShape* shape);

public:

	Vehicle(void);	
	~Vehicle(void);

	bool Create(std::string chasis,std::string wheel,btDynamicsWorld *dynamicsWorld, btVector3 &position, btScalar mass );		
	void Render();	
	void Update(SystemManager* systemManager, float elapsedTime );
	void Destroy();

	btScalar GetSpeed()				{ return mRaycastVehicle->getCurrentSpeedKmHour(); }
	btScalar GetSteeringValue()		{ return mSteeringValue; }
	btVector3 GetDirection()		{ return mForward; }
	btTransform &GetWorldTransform() { return mRaycastVehicle->getRigidBody()->getWorldTransform(); } 
};

#endif